==> open-xchange-documents-ui_7.8.1.orig.tar.bz2: open-xchange-documents-ui-7.8.1/ui/apps/io.ox/office/textframework/utils/listutils.js <==
/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
  * © 2016 OX Software GmbH, Germany. info@open-xchange.com
 *
 * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
 */
define('io.ox/office/textframework/utils/listutils', [
    'io.ox/office/tk/utils'
], function (Utils) {

    'use strict';

    // static class ListUtils =================================================

    var ListUtils = {};

    // methods ----------------------------------------------------------------

    /**
     * Formatting a specified number value into a string of specified format.
     * Supported values for the parameter 'format' are 'lowerletter', 'upperletter',
     * 'lowerroman' and 'upperroman'.
     *
     * @param {Number} value
     *  One-based number to be converted.
     *
     * @param {String} format
     *  The format, in which the number shall be converted.
     *
     * @returns {String}
     *  The converted specified value. Or the empty string, if no conversion
     *  was done.
     */
    ListUtils.formatNumber = function (value, format) {

        /**
         * Converts the passed value to upper-case Latin letters.
         *
         * @param {Number} value
         *  One-based number to be converted.
         *
         * @returns {String}
         *  The upper-case representation of the passed value in Latin letters.
         *  Counts from 'A', 'B', 'C', ..., 'Z', 'AA', 'BB', 'CC', and so on.
         */
        function convertToLetters(value) {

            var letter = String.fromCharCode(65 + ((value - 1) % 26)),
                count = Math.floor((value - 1) / 26) + 1;

            return Utils.repeatString(letter, count);
        }

        /**
         * Converts the passed value to upper-case Roman numbers.
         *
         * @param {Number} value
         *  One-based number to be converted.
         *
         * @returns {String}
         *  The upper-case Roman number representation of the passed value.
         */
        function convertToRoman(value) {

            var result = '',
                romanDigitsArr = ['M', 'D', 'C', 'L', 'X', 'V', 'I'],
                romanValArr = [1000, 500, 100,  50,  10,   5,   1];

            if (value > 0) {
                var index = 0;
                for (; index < 7; index++) {
                    while (value >= romanValArr[index]) {
                        result += romanDigitsArr[index];
                        value -= romanValArr[index];
                    }
                    var position = 7;
                    for (; position > index; position--) {
                        var tempVal = romanValArr[index] - romanValArr[position];
                        if ((romanValArr[position] < tempVal) && (tempVal <= value)) {
                            result += romanDigitsArr[position] + romanDigitsArr[index];
                            value -= tempVal;
                        }
                    }
                }
            }

            return result;
        }

        // checking the specified format
        switch (format.toLowerCase()) {
            case 'decimal':
                return value.toString();
            case 'lowerletter':
                return convertToLetters(value).toLowerCase();
            case 'upperletter':
                return convertToLetters(value);
            case 'lowerroman':
                return convertToRoman(value).toLowerCase();
            case 'upperroman':
                return convertToRoman(value);
        }

        Utils.error('Lists.formatNumber() - unknown number format: ' + format);

        return '';
    };

    /**
     * Auto detection of lists from a specified text.
     *
     * @param {String} text
     *  Possible numbering label text
     *
     * @returns {Object}
     *  An object containing the properties 'numberFormat', 'listStartValue'
     *  (numbered lists only) and symbol (bullet lists only). Allowed values
     *  for the numberFormat are 'bullet', 'decimal', 'upperRoman' and
     *  'lowerRoman'.
     *  If no list can be detected, an empty object is returned.
     */
    ListUtils.detectListSymbol = function (text) {

        var ret = {};

        /**
         * Parsing a specified text to detect a list with Roman number format.
         *
         * @param {String} text
         *  Possible numbering label text
         *
         * @returns {Object}
         *  An object containing the properties 'numberFormat' and 'startnumber'.
         *  Allowed values for the numberFormat are 'upperRoman' and 'lowerRoman'.
         *  If no list can be detected, an empty object is returned.
         */
        function parseRoman(text) {

            var romanSmallArr = ['m', 'd', 'c', 'l', 'x', 'v', 'i'],
                romanValArr = [1000, 500, 100,  50,  10, 5, 1],
                localRet = {},
                lowerText = text.toLowerCase(),
                startValue = 0,
                index = 0,
                lastValue = 1000;

            localRet.caps = (lowerText !== text);

            for (; index < text.length; ++index) {

                var char = lowerText.charAt(index);
                if (char === '.') {
                    break;
                }
                var position = $.inArray(char, romanSmallArr);
                if (position < 0) {
                    return {};
                }
                var value = romanValArr[position];
                if (lastValue < value) {
                    startValue = startValue - lastValue + (value - lastValue);
                } else {
                    startValue += value;
                }
                lastValue = value;
            }

            if (startValue > 0) {
                localRet.startnumber = startValue;
                localRet.numberFormat = lowerText !== text ? 'upperRoman' : 'lowerRoman';
            }

            return localRet;
        }

        // searching in the specified string for '-', '*' or '.'

        if (text.length === 1 && (text === '-' || text === '*')) {
            // bullet
            ret.numberFormat = 'bullet';
            ret.symbol = text;
        } else if (text.substring(text.length - 1) === '.') {
            var sub = text.substring(0, text.length - 1);
            var startnumber = parseInt(sub, 10);
            if (startnumber > 0) {
                ret.numberFormat = 'decimal';
                ret.listStartValue = startnumber;
            } else {
                var roman = parseRoman(text);
                if (roman.startnumber > 0) {
                    ret.numberFormat = roman.numberFormat;
                    ret.listStartValue = roman.startnumber;
                }
            }
        }

        return ret;
    };

    // exports ================================================================

    return ListUtils;

});

==> open-xchange-documents-ui_7.8.2.orig.tar.bz2: open-xchange-documents-ui-7.8.2/ui/apps/io.ox/office/textframework/utils/listutils.js <==
/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
  * © 2016 OX Software GmbH, Germany. info@open-xchange.com
 *
 * @author Ingo Schmidt-Rosbiegal <ingo.schmidt-rosbiegal@open-xchange.com>
 */
define('io.ox/office/textframework/utils/listutils', [
    'io.ox/office/tk/utils'
], function (Utils) {

    'use strict';

    // static class ListUtils =================================================

    var ListUtils = {};

    // methods ----------------------------------------------------------------

    /**
     * Formatting a specified number value into a string of specified format.
     * Supported values for the parameter 'format' are 'lowerletter', 'upperletter',
     * 'lowerroman' and 'upperroman'.
     *
     * @param {Number} value
     *  One-based number to be converted.
     *
     * @param {String} format
     *  The format, in which the number shall be converted.
     *
     * @returns {String}
     *  The converted specified value. Or the empty string, if no conversion
     *  was done.
     */
    ListUtils.formatNumber = function (value, format) {

        /**
         * Converts the passed value to upper-case Latin letters.
         *
         * @param {Number} value
         *  One-based number to be converted.
         *
         * @returns {String}
         *  The upper-case representation of the passed value in Latin letters.
         *  Counts from 'A', 'B', 'C', ..., 'Z', 'AA', 'BB', 'CC', and so on.
         */
        function convertToLetters(value) {

            var letter = String.fromCharCode(65 + ((value - 1) % 26)),
                count = Math.floor((value - 1) / 26) + 1;

            return Utils.repeatString(letter, count);
        }

        /**
         * Converts the passed value to upper-case Roman numbers.
         *
         * @param {Number} value
         *  One-based number to be converted.
         *
         * @returns {String}
         *  The upper-case Roman number representation of the passed value.
         */
        function convertToRoman(value) {

            var result = '',
                romanDigitsArr = ['M', 'D', 'C', 'L', 'X', 'V', 'I'],
                romanValArr = [1000, 500, 100,  50,  10,   5,   1];

            if (value > 0) {
                var index = 0;
                for (; index < 7; index++) {
                    while (value >= romanValArr[index]) {
                        result += romanDigitsArr[index];
                        value -= romanValArr[index];
                    }
                    var position = 7;
                    for (; position > index; position--) {
                        var tempVal = romanValArr[index] - romanValArr[position];
                        if ((romanValArr[position] < tempVal) && (tempVal <= value)) {
                            result += romanDigitsArr[position] + romanDigitsArr[index];
                            value -= tempVal;
                        }
                    }
                }
            }

            return result;
        }

        // checking the specified format
        switch (format.toLowerCase()) {
            case 'decimal':
                return value.toString();
            case 'lowerletter':
                return convertToLetters(value).toLowerCase();
            case 'upperletter':
                return convertToLetters(value);
            case 'lowerroman':
                return convertToRoman(value).toLowerCase();
            case 'upperroman':
                return convertToRoman(value);
        }

        Utils.error('Lists.formatNumber() - unknown number format: ' + format);

        return '';
    };

    /**
     * Auto detection of lists from a specified text.
     *
     * @param {String} text
     *  Possible numbering label text
     *
     * @returns {Object}
     *  An object containing the properties 'numberFormat', 'listStartValue'
     *  (numbered lists only) and symbol (bullet lists only). Allowed values
     *  for the numberFormat are 'bullet', 'decimal', 'upperRoman' and
     *  'lowerRoman'.
     *  If no list can be detected, an empty object is returned.
     */
    ListUtils.detectListSymbol = function (text) {

        var // the return object used for collecting list information
            ret = {};

        /**
         * Parsing a specified text to detect a list with Roman number format.
         *
         * @param {String} text
         *  Possible numbering label text
         *
         * @returns {Object}
         *  An object containing the properties 'numberFormat' and 'startnumber'.
         *  Allowed values for the numberFormat are 'upperRoman' and 'lowerRoman'.
         *  If no list can be detected, an empty object is returned.
         */
        function parseRoman(text) {

            var romanSmallArr = ['m', 'd', 'c', 'l', 'x', 'v', 'i'],
                romanValArr = [1000, 500, 100,  50,  10, 5, 1],
                localRet = {},
                lowerText = text.toLowerCase(),
                startValue = 0,
                index = 0,
                lastValue = 1000;

            localRet.caps = (lowerText !== text);

            for (; index < text.length; ++index) {

                var char = lowerText.charAt(index);
                if (char === '.') {
                    break;
                }
                var position = $.inArray(char, romanSmallArr);
                if (position < 0) {
                    return {};
                }
                var value = romanValArr[position];
                if (lastValue < value) {
                    startValue = startValue - lastValue + (value - lastValue);
                } else {
                    startValue += value;
                }
                lastValue = value;
            }

            if (startValue > 0) {
                localRet.startnumber = startValue;
                localRet.numberFormat = lowerText !== text ? 'upperRoman' : 'lowerRoman';
            }

            return localRet;
        }

        // searching in the specified string for '-', '*' or '.'

        if (text.length === 1 && (text === '-' || text === '*')) {
            // bullet
            ret.numberFormat = 'bullet';
            ret.symbol = text;
        } else if (text.substring(text.length - 1) === '.' || text.substring(text.length - 1) === ')') {
            var first = text.substring(0, 1);
            var sub = text.substring(0, text.length - 1);
            if (first === '(') { sub = text.substring(1, text.length - 1); }
            var startnumber = parseInt(sub, 10);
            if (startnumber > 0) {
                ret.numberFormat = 'decimal';
                ret.listStartValue = startnumber;
                ret.leftString = (first === '(') ? first : '';
                ret.rightString = text.substring(text.length - 1);
            } else {
                var roman = parseRoman(text);
                if (roman.startnumber > 0) {
                    ret.numberFormat = roman.numberFormat;
                    ret.listStartValue = roman.startnumber;
                    ret.leftString = '';
                    ret.rightString = text.substring(text.length - 1);
                }
            }
        }

        return ret;
    };

    // exports ================================================================

    return ListUtils;

});
